import maya.cmds as mc
from maya import OpenMayaUI as omui
import maya.api.OpenMaya as OpenMaya

try:
    # noinspection PyUnresolvedReferences
    from shiboken2 import wrapInstance
except ImportError:
    # noinspection PyUnresolvedReferences
    from shiboken import wrapInstance

# noinspection PyUnresolvedReferences
# pylint: disable=no-name-in-module
from mocapx.vendor.Qt import QtGui, QtWidgets, QtCore
from mocapx.lib.utils import has_attr_connection, is_node_anim_curve, is_node_expression, \
    is_node_constraint, is_node_pair_blend

# pylint: enable=no-name-in-module

COLOR_GROUPS = [
    "Disabled",
    "Active",
    "Inactive"]

COLOR_ROLES = [
    "Window",
    "Background",
    "WindowText",
    "Foreground",
    "Base",
    "AlternateBase",
    "ToolTipBase",
    "ToolTipText",
    "Text",
    "Button",
    "ButtonText",
    "BrightText",
    "Light",
    "Midlight",
    "Dark",
    "Shadow",
    "Highlight",
    "HighlightedText",
    "Link",
    "LinkVisited",
    "NoRole"]


class RegExValidator(QtGui.QRegExpValidator):
    def __init__(self, *args, **kwargs):
        super(RegExValidator, self).__init__(*args, **kwargs)

    def validate(self, _input, _pos):
        state = super(RegExValidator, self).validate(_input, _pos)
        if state[0] == QtGui.QValidator.State.Intermediate:
            return QtGui.QValidator.State.Acceptable, state[1], state[2]
        return state


def is_widget_minimized(widget):
    if widget.isMinimized():
        return True
    parent = widget.parent()
    if parent:
        return is_widget_minimized(parent)
    return False


def is_widget_really_visible(widget):
    if widget.isHidden():
        return False
    if widget.visibleRegion().isEmpty():
        return False
    if is_widget_minimized(widget):
        return False

    # is widget outside the screens area
    desktop = QtWidgets.QApplication.desktop()
    for screen in range(desktop.screenCount()):
        screen_rect = desktop.screenGeometry(screen)
        widget_rect = widget.geometry()
        widget_rect.moveTopLeft(widget.mapToGlobal(widget_rect.topLeft()))
        if screen_rect.intersects(widget_rect):
            return True
    return False


def update_indicator_color(plug, widget, class_name="QLineEdit", border=True):
    if mc.getAttr(plug, lock=True):
        color = "color: black; background-color: rgb(92, 104, 116)"
    elif has_attr_connection(plug):
        incoming = mc.listConnections(plug, d=False, scn=True)[0]
        if mc.objectType(incoming, i="mute"):
            color = "color: black; background-color: rgb(191, 161, 130)"
        elif is_node_anim_curve(incoming, subtype="T"):
            color = "color: rgb(187, 187, 165); background-color: rgb(205, 39, 41)"
        elif is_node_anim_curve(incoming, subtype="U"):
            color = "color: black; background-color: rgb(80, 153, 218)"
        elif is_node_expression(incoming):
            color = "color: black; background-color: rgb(203, 165, 241)"
        elif is_node_constraint(incoming):
            color = "color: black; background-color: rgb(163, 203, 240)"
        elif is_node_pair_blend(incoming):
            color = "color: black; background-color: rgb(172, 241, 172)"
        else:
            color = "color: black; background-color: rgb(241, 241, 165)"
    else:
        color = ""

    if class_name:
        color = class_name + " {" + "border: none; border-radius: 2px;" * (not border) + color + "}"

    widget.setStyleSheet(color)


def draw_rounded_rect(painter, widget, margin, angle_factor):
    w = widget.width() - margin * 2
    h = widget.height() - margin * 2
    painter.drawRoundedRect(margin, margin, w, h, h * angle_factor, h * angle_factor)


def get_application_palette():
    # on Maya 2018 no service pack windows QtWidgets.QApplication.palette()
    # fails with the error : palette() takes exactly one argument (0 given)
    # so in case of the error we are trying to submit None as an argument
    try:
        palette = QtWidgets.QApplication.palette()
    except TypeError as er:
        palette = QtWidgets.QApplication.palette(None)
    return palette


def get_color_from_maya_palette(c_role, c_group="Active", palette=None, getRgb=False):
    if not palette:
        palette = get_application_palette()

    q_role = getattr(QtGui.QPalette, c_role)
    q_group = getattr(QtGui.QPalette, c_group)
    color = palette.color(q_group, q_role)

    if getRgb:
        color = color.getRgb()

    return color


def get_maya_palette(palette=None, getRgb=False):
    if not palette:
        palette = get_application_palette()

    p_colors = {}

    for c_role in COLOR_ROLES:
        for c_group in COLOR_GROUPS:
            p_colors[c_role + "." + c_group] = get_color_from_maya_palette(c_role, c_group, palette, getRgb)

    return p_colors


def set_color_palette(widget, c_role, q_color, c_group="Active", reset_palette=False):
    q_role = getattr(QtGui.QPalette, c_role)
    q_group = getattr(QtGui.QPalette, c_group)

    palette = QtGui.QPalette() if reset_palette else widget.palette()
    palette.setColor(q_group, q_role, q_color)

    widget.setAutoFillBackground(True)
    widget.setPalette(palette)


def set_color_palette_lighter(widget, c_role, factor, c_group="Active", reset_palette=False):
    palette = QtGui.QPalette() if reset_palette else widget.palette()
    q_color = get_color_from_maya_palette(c_role, c_group, palette).lighter(factor)
    set_color_palette(widget, c_role, q_color, c_group)


def set_color_palette_darker(widget, c_role, factor, c_group="Active", reset_palette=False):
    palette = QtGui.QPalette() if reset_palette else widget.palette()
    q_color = get_color_from_maya_palette(c_role, c_group, palette).darker(factor)
    set_color_palette(widget, c_role, q_color, c_group)


def get_widget(widget_name):
    ptr = omui.MQtUtil.findLayout(widget_name)
    widget = wrapInstance(int(ptr), QtWidgets.QWidget)
    return widget


def create_attr_outliner(select_command=None, close_command=None, state_option=None):
    pane = mc.window()
    if close_command:
        mc.window(pane, e=True, closeCommand=close_command)

    if state_option and mc.optionVar(exists=state_option):
        mc.window(pane, e=True, state=mc.optionVar(q=state_option))

    form = mc.formLayout(numberOfDivisions=100)
    outliner = mc.nodeOutliner(
        showOutputs=True,
        showNonKeyable=True,
        showHidden=False,
        showNonConnectable=False)

    if select_command:
        mc.nodeOutliner(outliner, e=True, selectCommand=select_command)

    mc.formLayout(
        form,
        edit=True,
        attachForm=[
            (outliner, "top", 5),
            (outliner, "left", 5),
            (outliner, "bottom", 5),
            (outliner, "right", 5)])

    return pane, outliner


def window_exists(window_name):
    return mc.window(window_name, exists=True)


def set_connected_plug(outliner, attached_plug):
    mc.nodeOutliner(outliner, e=True, connectivity=attached_plug)


def set_connected_node(outliner, node):
    mc.nodeOutliner(outliner, e=True, replace=node)


def selection(outliner):
    return mc.nodeOutliner(outliner, q=True, lastClickedNode=True)


def show(window):
    mc.showWindow(window)


def confirm_dialog(*args, **kwargs):
    return mc.confirmDialog(*args, **kwargs)


# noinspection PyCallByClass
def create_sel_changed_cb(cd):
    cbid = OpenMaya.MEventMessage.addEventCallback("SelectionChanged", cd, clientData=None)
    return cbid


def remove_cb(cbid):
    OpenMaya.MMessage.removeCallback(cbid)


def set_window_title(window, title):
    return mc.window(window, e=True, title=title)


def save_window_state(window):
    return mc.window(window, q=True, state=True)


def load_window_state(window, state_info):
    return mc.window(window, e=True, state=state_info)


def get_maya_window():
    maya_main_window_ptr = omui.MQtUtil.mainWindow()
    # noinspection PyCompatibility
    return wrapInstance(long(maya_main_window_ptr), QtWidgets.QWidget)


def ask_user_confirm(message):
    answer = mc.confirmDialog(
        title='Confirm',
        message=message,
        button=('Continue', 'Cancel'),
        defaultButton='Continue',
        cancelButton='Cancel',
        dismissString='Continue')

    if answer == 'Continue':
        return True
    else:
        return False


def show_warning(msg):
    mc.warning(msg)


def show_error(msg):
    OpenMaya.MGlobal.displayError(msg)


class DebugPane(QtWidgets.QDialog):
    def __init__(self, widget, parent=None):
        super(DebugPane, self).__init__(parent)
        self.widget = widget
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.widget)
        self.setLayout(layout)

    # noinspection PyPep8Naming,PyUnusedLocal
    def closeEvent(self, event):
        self.widget.clean_callbacks()


def build_maya_pane(widget):
    return DebugPane(widget, parent=get_maya_window())
